from listcrunch import crunch, uncrunch

def test_basic_crunch():
    assert crunch([1,2,3]) == '1:0;2:1;3:2'

def test_basic_uncrunch():
    assert uncrunch('1:0;2:1;3:2') == [1,2,3]

def test_basic_run():
    assert crunch([1,1,1]) == '1:0-2'

def test_basic_run_uncrunch():
    assert uncrunch('1:0-2') == [1,1,1]

def test_basic_broken_run():
    assert crunch([1,2,1]) == '1:0,2;2:1'

def test_basic_broken_run_uncrunch():
    assert uncrunch('1:0,2;2:1') == [1,2,1]

def test_order():
    assert crunch([2,1]) == '2:0;1:1'

def test_order_uncrunch():
    assert uncrunch('2:0;1:1') == [2,1]

def test_advanced():
    assert crunch([50, 50, 3, 50, 50, 3, 60, 70, 70]) == '50:0-1,3-4;3:2,5;60:6;70:7-8'

def test_advanced_uncrunch():
    assert uncrunch('50:0-1,3-4;3:2,5;60:6;70:7-8') == [50, 50, 3, 50, 50, 3, 60, 70, 70]

def test_empty_crunch():
    assert crunch([]) == ''

def test_empty_uncrunch():
    assert uncrunch('') == []

def test_single_item_crunch():
    assert crunch([77]) == '77:0'

def test_single_item_uncrunch():
    assert uncrunch('77:0') == [77]



import collections

def crunch(items):
    cruncher = collections.defaultdict(list)
    for num, item in enumerate(items):
        cruncher[item].append(num)
    
    parts = []
    for value in cruncher:
        nums = sorted(cruncher[value])

        subparts = []
        run = None

        def end_run(run):
            if run is None:
                return
            if run[0] == run[1]:
                subparts.append(f"{run[0]}")
            else:
                subparts.append(f"{run[0]}-{run[1]}")
        
        for num in nums:
            if run is None:
                run = [num, num]
            else:
                if num == run[1] + 1:
                    run[1] = num
                else:
                    end_run(run)
                    # Start a new run
                    run = [num, num]

        end_run(run)
        joined = ",".join(subparts)
        parts.append(f"{value}:{joined}")

    return ";".join(parts)

def uncrunch(s):
    if len(s.strip()) == 0: return []

    results = []

    parts = s.split(';')
    for part in parts:
        subparts = part.split(':')
        assert len(subparts) == 2
        value = float(subparts[0])
        specs = subparts[1].split(',')

        for spec in specs:
            if '-' in spec:
                # Parse a range
                startEnd = spec.split('-')
                assert len(startEnd) == 2
                start = int(startEnd[0])
                end = int(startEnd[1])
                for i in range(start, end + 1):
                    results.append([i, value])
            else:
                results.append([int(spec), value])
    
    sorted_results = sorted(results, key=lambda x: x[0])
    nums, values = zip(*sorted_results)
    assert list(nums) == list(range(len(results)))
    return list(values)